#!/usr/bin/ruby -w

require 'colorize'
String.disable_colorization false

require 'datafarming/error_handling'
require 'datafarming/freq_sets'

help_msg = [
  'Generate scaled FDE designs with shifting and stacking. The resulting',
  'design is written to ' +
    'stdout'.light_blue + '.', '',
  'Syntax:',
  "\n\t#{$PROGRAM_NAME.split(%r{/|\\})[-1]} [--help]".yellow +
    " [-c] [-d #] [-e] [-k #] [-s #] [file_name]\n".yellow,
  "Arguments in square brackets are optional.  A vertical bar '|'",
  'indicates valid alternatives for invoking the option.  Prefix',
  'the command with "' + 'ruby'.yellow +
    '" if it is not on your PATH.', '',
  '  --help | -h | -? | ?'.green,
  "\tProduce this help message.  Supersedes any other choices.",
  '  --csv | -c'.green,
  "\tGenerate output as comma-separated-values.",
  '  --num_factors # | -k #'.green,
  "\tSpecify number of factors using standard (+/-1) scaling.",
  '  --design # | -d #'.green,
  "\tSpecify which design number you want.  Default is highest",
  "\tfrequency set available for the target number of factors.",
  "\tAn input larger than the available number will produce an",
  "\terror message that identifies the allowable range.",
  '  --stack # | -s #'.green,
  "\t# specifies the number of stackings. The specified value cannot",
  "\texceed the number of columns in the design being used. If this",
  "\toption is omitted then only the base design is generated.",
  '  --excel-style-input | -e'.green,
  "\tSpecify factor ranges similarly to the NOLH spreadsheet, i.e.,",
  "\tthe first line is the set of minimum range values for all factors,",
  "\tand the second line is maximum range values.  Without this option,",
  "\tthe default input format is one line per factor, comprised of the",
  "\tmin and max separated by commas or whitespace.",
  '  file_name'.green,
  "\tThe name of a file containing the factor specifications.  If no",
  "\tfilename is given, the user can enter the values interactively in",
  "\tthe desired form or use file redirection with '<'.", '',
  'Options may be given in any order, but must come before the file name',
  'if one is provided.'
]

# Scaler objects will rescale a FDE design from standard units
# to a range specified by min and max
class Scaler
  def initialize(min = -1, max = 1)
    @mid = 0.5 * (max + min)
    @amplitude = 0.5 * (max - min)
  end

  def scale(value)
    @mid + @amplitude * value
  end
end

def make_design(number_of_factors, design_num = -1)
  design = Array.new(number_of_factors)
  ds = FDE::DESIGN_SETS[number_of_factors]
  freqs = ds.freqs[design_num].map { |f| Rational(f, ds.nyq) }
  design.each_index do |column|
    omega = freqs[column]
    design[column] = Array.new(ds.nyq) do |i|
      Math.sin(((i * omega) % 1) * FDE::TWO_PI)
    end
  end
  design.transpose
end

excel_style_inputs = false
csv_out = false
design_num = -1
num_stackings = 1
n = nil
while ARGV[0] && (ARGV[0][0] == '-' || ARGV[0][0] == 45 || ARGV[0][0] == '?')
  current_value = ARGV.shift
  case current_value
  when '--csv', '-c'
    csv_out = true
  when '--design', '-d'
    design_num = ARGV.shift.to_i - 1
  when '--num_factors', '-k'
    n = ARGV.shift.to_i
  when '--stack', '-s'
    num_stackings = ARGV.shift.to_i
  when '--excel-style-input', '-e'
    excel_style_inputs = true
  when '--help', '-h', '-help', '-?', '?'
    ErrorHandling.clean_abort help_msg
  else
    ErrorHandling.message ['Unknown argument: '.red + current_value.yellow]
    ErrorHandling.clean_abort help_msg
  end
end

if n
  factor = Array.new(n) { Scaler.new }
else
  begin
    if excel_style_inputs
      if ARGF.filename == '-'
        STDERR.puts  'Enter one line of min values, and one of max values.'.green
      end
      min_values = ARGF.gets.strip.split(/\s*[,;:]\s*|\s+/).map(&:to_f)
      max_values = ARGF.gets.strip.split(/\s*[,;:]\s*|\s+/).map(&:to_f)
    else
      if ARGF.filename == '-'
        STDERR.puts  'To terminate input enter '.green + 'ctrl-d'.cyan +
                    ' (Mac/Unix/Linux)'.green + ' or '.green + 'ctrl-z'.cyan +
                    ' (Windows).'.green
        STDERR.puts  'Enter ranges for each factor on a separate line.'.green
        STDERR.puts  "\nMIN\tMAX".cyan
      end
      min_values = []
      max_values = []
      while line = ARGF.gets
        values = line.strip.split(/\s*[,;:]\s*|\s+/).map(&:to_f)
        min_values << values.shift
        max_values << values.shift
      end
    end
  rescue StandardError => e
    ErrorHandling.message [e.message.red]
    ErrorHandling.clean_abort help_msg
  end

  n = min_values.size
  if max_values.size != n
    ErrorHandling.message ['Unequal counts for min, max'.red]
    ErrorHandling.clean_abort help_msg
  end

  factor = Array.new(n) do |i|
    Scaler.new(min_values[i], max_values[i])
  end
end

if design_num >= FDE::DESIGN_SETS[n].freqs.length
  ErrorHandling.clean_abort [
    "Design number cannot exceed #{FDE::DESIGN_SETS[n].freqs.length} for #{n} factors".red
  ]
end

design = make_design(n, design_num)

num_columns = design[0].length
if num_stackings > num_columns
  ErrorHandling.clean_abort [
    'Requested stacking exceeds number of columns in FDE '.red +
    "(#{num_columns})".red
  ]
end

num_stackings.times do |_stack_num|
  design.each_with_index do |dp, i|
    scaled_dp = dp.slice(0, n).map.with_index { |x, k| factor[k].scale(x) }
    if csv_out
      puts scaled_dp.join ','
    else
      puts scaled_dp.join "\t"
    end
    design[i] = dp.rotate
  end
end
