#!/usr/bin/ruby -w

require 'colorize'
String.disable_colorization false

require_relative '../lib/datafarming/error_handling'
require_relative '../lib/datafarming/factorial_generator'

def print_descriptor
  description = [
    "\nCreate a Resolution V fractional factorial design, either in standard +/-1",
    'notation or scaled as per user-provided input. The design can optionally ',
    'be supplemented with CCD center and star points.  Output is written to',
    'stdout'.blue + ' with formatting controlled by command-line options.'
  ]
  STDERR.puts description
end

def print_directions # :nodoc:
  prog = $PROGRAM_NAME.split(/\//)[-1]
  help_msg = [
    '', 'Syntax:', '',
    "  ruby #{prog} [Options] #factors | low/high-settings".yellow, '',
    "The vertical bar '|' indicates valid alternatives.", '',
    'Command-line args must specify either:',
    '  - the number of factors for the design; or',
    '  - low and hi values for all factors.', '',
    "\nOptions:",
    '  --help | -h | -? | ?'.green,
    "\tProduce this help message.",
    '  --design-only | -d'.green,
    "\tprint the design without labeling",
    '  --csv | -c'.green,
    "\tprint the design as comma separated",
    '  --center | -a'.green,
    "\tadd global center pt to check linearity",
    '  --star | -s'.green,
    "\taugment with Central Composite Design pts (includes --center)",
    '  --rotatable | -r'.green,
    "\tscale a CCD design to be rotatable (includes --star)",
    ''
  ]
  STDERR.puts help_msg
end

(print_descriptor && print_directions && exit) if ARGV.empty?
begin
  labels = true
  csv = false
  scaled = false
  center = false
  star = false
  rotatable = false
  fmt_size = '5'
  fmt_type = 'd'

  while ARGV[0] =~ /^(-\D|\?)/
    arg = ARGV.shift
    case arg
    when '-c', '--csv'
      csv = true
    when '-d', '--design-only'
      labels = false
    when '-a', '--center'
      center = true
    when '-s', '--star'
      star = true
      center = true
    when '-r', '--rotatable'
      rotatable = true
      star = true
      center = true
      fmt_size = '11.8'
      fmt_type = 'f'
    when '-h', '--help', '-?', '?'
      print_descriptor
      print_directions
      exit
    else
      raise "Unknown option #{arg}"
    end
  end

  if ARGV.empty?
    raise 'Must specify a design size'
  elsif ARGV.length == 1
    n = ARGV.shift.to_i
  elsif (ARGV.length & 1).nonzero? # Odd number of args remaining!
    raise 'Number of levels is odd: low/hi values must occur in pairs'
  else
    n = ARGV.length / 2
    scaled = true
    unless rotatable
      fmt_type = 's'
      fmt_size = [ARGV.map(&:length).max + 1, fmt_size.to_i].max
    end
  end

  my_design = make_design n
  if rotatable && star
    inverse_len = 1.0 / Math.sqrt(n)
    my_design.each_with_index do |row, i|
      my_design[i] = row.map { |elt| elt * inverse_len }
    end
  end
  my_design += [Array.new(n, 0)] if center

  if star
    star_design = Array.new(2 * n)
    star_design.each_index { |row| star_design[row] = Array.new(n, 0) }
    n.times do |col|
      row = 2 * col
      star_design[row][col] = -1
      star_design[row + 1][col] = 1
    end
    my_design += star_design
  end

  if scaled
    temp = my_design.transpose
    temp.each_with_index do |factor, idx|
      factor.map! do |value|
        case value
        when -1
          ARGV[2 * idx + 1]
        when 1
          ARGV[2 * idx]
        when 0
          begin
            0.5 * (Float(ARGV[2 * idx + 1]) + Float(ARGV[2 * idx]))
          rescue
            raise 'All factors must be numeric for augmented designs.'
          end
        else
          begin
            lo = Float(ARGV[2 * idx])
            hi = Float(ARGV[2 * idx + 1])
            range = hi - lo
            0.5 * (hi + lo + value * range)
          rescue
            raise 'All factors must be numeric for augmented designs.'
          end
        end
      end
    end
    my_design = temp.transpose
  end

  if labels
    if csv
      print ' DP#'
      n.times { |i| print ",X#{i + 1}" }
    else
      print '  DP#'
      n.times { |i| printf " %#{fmt_size}s", "X#{i + 1}" }
    end
    puts
  end

  my_design.each_with_index do |design_point, row_number|
    if csv
      printf('%d,', row_number + 1) if labels
      puts design_point.join(',')
    else
      printf('%5d', row_number + 1) if labels
      design_point.each { |value| printf " %#{fmt_size}#{fmt_type}", value }
      puts
    end
  end
rescue StandardError => e
  STDERR.print "\n", e.message, "\n"
  print_directions
end
